MAKEFILE_LIST := $(patsubst $(lastword $(MAKEFILE_LIST)),,$(MAKEFILE_LIST))
LINUX := $(call UPPERCASE,$(call pkgname))

$(LINUX)_LICENSE = GPLv2
$(LINUX)_LICENSE_FILES = COPYING
$(LINUX)_PKGDIR  := $(call pkgdir)
$(LINUX)_RAWVERSION := $($(LINUX)_MAJOR_VERSION)$($(LINUX)_MINOR_VERSION)
$(LINUX)_EXTRAVERSION := $(if $($(LINUX)_FLAVOUR),-$($(LINUX)_FLAVOUR))
$(LINUX)_EXT_VERSION := $($(LINUX)_RAWVERSION)$(if $($(LINUX)_MINOR_VERSION),,.0)
$(LINUX)_VERSION := $($(LINUX)_EXT_VERSION)$($(LINUX)_EXTRAVERSION)
$(LINUX)_MODULES :=

# Compute $(LINUX)_SOURCE and $(LINUX)_SITE from the configuration
$(LINUX)_SOURCE := linux-$($(LINUX)_MAJOR_VERSION).tar.xz
# In X.Y.Z, get X and Y. We replace dots and dashes by spaces in order
# to use the $(word) function. We support versions such as 3.1,
# 2.6.32, 2.6.32-rc1, 3.0-rc6, etc.
ifneq ($(filter 2.6.%,$($(LINUX)_VERSION)),)
$(LINUX)_SITE := $(BR2_KERNEL_MIRROR)/linux/kernel/v2.6
else
$(LINUX)_SITE := $(BR2_KERNEL_MIRROR)/linux/kernel/v3.x
endif
# release candidates are in testing/ subdir
ifneq ($(findstring -rc,$($(LINUX)_VERSION)),)
$(LINUX)_SITE += /testing
endif # -rc

ifneq ($($(LINUX)_MINOR_VERSION),)
$(LINUX)_PATCH := $($(LINUX)_SITE)/patch-$($(LINUX)_RAWVERSION).xz
endif

$(LINUX)_PATCHES := $(call qstrip,$(BR2_LINUX_KERNEL_PATCH))

$(LINUX)_INSTALL_IMAGES := NO
$(LINUX)_DEPENDENCIES := host-kmod host-lzop

ifeq ($(BR2_LINUX_KERNEL_UBOOT_IMAGE),y)
	$(LINUX)_DEPENDENCIES += host-uboot-tools
endif

$(LINUX)_MAKE_FLAGS = \
	HOSTCC="$(HOSTCC)" \
	HOSTCFLAGS="$(HOSTCFLAGS)" \
	ARCH=$(KERNEL_ARCH) \
	INSTALL_MOD_PATH=$(TARGET_DIR) \
	INSTALL_FW_PATH=$(TARGET_DIR)/lib/modules/$($(LINUX)_VERSION)/firmware \
	CROSS_COMPILE="$(CCACHE) $(TARGET_CROSS)" \
	DEPMOD=$(HOST_DIR)/sbin/depmod \
	EXTRAVERSION=$($(LINUX)_EXTRAVERSION)

# Get the real Linux version, which tells us where kernel modules are
# going to be installed in the target filesystem.
$(LINUX)_VERSION_PROBED = $(shell $(MAKE) $($(LINUX)_MAKE_FLAGS) -C $($(LINUX)_DIR) --no-print-directory -s kernelrelease)

ifeq ($(BR2_LINUX_KERNEL_USE_INTREE_DTS),y)
KERNEL_DTS_NAME = $(call qstrip,$(BR2_LINUX_KERNEL_INTREE_DTS_NAME))
else ifeq ($(BR2_LINUX_KERNEL_USE_CUSTOM_DTS),y)
KERNEL_DTS_NAME = $(basename $(notdir $(call qstrip,$(BR2_LINUX_KERNEL_CUSTOM_DTS_PATH))))
endif

ifeq ($(BR2_LINUX_KERNEL_DTS_SUPPORT)$(KERNEL_DTS_NAME),y)
$(error No kernel device tree source specified, check your \
BR2_LINUX_KERNEL_USE_INTREE_DTS / BR2_LINUX_KERNEL_USE_CUSTOM_DTS settings)
endif

ifeq ($(BR2_LINUX_KERNEL_APPENDED_DTB),y)
ifneq ($(words $(KERNEL_DTS_NAME)),1)
$(error Kernel with appended device tree needs exactly one DTS source. \
	Check BR2_LINUX_KERNEL_INTREE_DTS_NAME or BR2_LINUX_KERNEL_CUSTOM_DTS_PATH.)
endif
endif

KERNEL_DTBS = $(addsuffix .dtb,$(KERNEL_DTS_NAME))

ifeq ($(BR2_LINUX_KERNEL_IMAGE_TARGET_CUSTOM),y)
$(LINUX)_IMAGE_NAME = $(call qstrip,$(BR2_LINUX_KERNEL_IMAGE_NAME))
$(LINUX)_TARGET_NAME = $(call qstrip,$(BR2_LINUX_KERNEL_IMAGE_TARGET_NAME))
else
ifeq ($(BR2_LINUX_KERNEL_UIMAGE),y)
$(LINUX)_IMAGE_NAME = uImage
else ifeq ($(BR2_LINUX_KERNEL_APPENDED_UIMAGE),y)
$(LINUX)_IMAGE_NAME = uImage
else ifeq ($(BR2_LINUX_KERNEL_BZIMAGE),y)
$(LINUX)_IMAGE_NAME = bzImage
else ifeq ($(BR2_LINUX_KERNEL_ZIMAGE),y)
$(LINUX)_IMAGE_NAME = zImage
else ifeq ($(BR2_LINUX_KERNEL_APPENDED_ZIMAGE),y)
$(LINUX)_IMAGE_NAME = zImage
else ifeq ($(BR2_LINUX_KERNEL_CUIMAGE),y)
$(LINUX)_IMAGE_NAME = cuImage.$(KERNEL_DTS_NAME)
else ifeq ($(BR2_LINUX_KERNEL_SIMPLEIMAGE),y)
$(LINUX)_IMAGE_NAME = simpleImage.$(KERNEL_DTS_NAME)
else ifeq ($(BR2_LINUX_KERNEL_LINUX_BIN),y)
$(LINUX)_IMAGE_NAME = linux.bin
else ifeq ($(BR2_LINUX_KERNEL_VMLINUX_BIN),y)
$(LINUX)_IMAGE_NAME = vmlinux.bin
else ifeq ($(BR2_LINUX_KERNEL_VMLINUX),y)
$(LINUX)_IMAGE_NAME = vmlinux
else ifeq ($(BR2_LINUX_KERNEL_VMLINUZ),y)
$(LINUX)_IMAGE_NAME = vmlinuz
endif
$(LINUX)_TARGET_NAME = $($(LINUX)_IMAGE_NAME)
endif

ifeq ($($(LINUX)_IMAGE_NAME),)
$(LINUX)_IMAGE_NAME = $($(LINUX)_TARGET_NAME)
endif

$(LINUX)_KERNEL_UIMAGE_LOADADDR = $(call qstrip,$(BR2_LINUX_KERNEL_UIMAGE_LOADADDR))
ifneq ($(LINUX_KERNEL_UIMAGE_LOADADDR),)
$(LINUX)_MAKE_FLAGS += LOADADDR="$(LINUX_KERNEL_UIMAGE_LOADADDR)"
endif

# Compute the arch path, since i386 and x86_64 are in arch/x86 and not
# in arch/$(KERNEL_ARCH). Even if the kernel creates symbolic links
# for bzImage, arch/i386 and arch/x86_64 do not exist when copying the
# defconfig file.
ifeq ($(KERNEL_ARCH),i386)
$(LINUX)_KERNEL_ARCH_PATH = $($(PKG)_DIR)/arch/x86
else ifeq ($(KERNEL_ARCH),x86_64)
$(LINUX)_KERNEL_ARCH_PATH = $($(PKG)_DIR)/arch/x86
else
$(LINUX)_KERNEL_ARCH_PATH = $($(PKG)_DIR)/arch/$(KERNEL_ARCH)
endif

ifeq ($(BR2_LINUX_KERNEL_VMLINUX),y)
$(LINUX)_IMAGE_PATH = $($(PKG)_DIR)/$($(PKG)_IMAGE_NAME)
else ifeq ($(BR2_LINUX_KERNEL_VMLINUZ),y)
$(LINUX)_IMAGE_PATH = $($(PKG)_DIR)/$($(PKG)_IMAGE_NAME)
else
ifeq ($(KERNEL_ARCH),avr32)
$(LINUX)_IMAGE_PATH = $($(PKG)_KERNEL_ARCH_PATH)/boot/images/$($(PKG)_IMAGE_NAME)
else
$(LINUX)_IMAGE_PATH = $($(PKG)_KERNEL_ARCH_PATH)/boot/$($(PKG)_IMAGE_NAME)
endif
endif # BR2_LINUX_KERNEL_VMLINUX

define $(LINUX)_DOWNLOAD_PATCH
	$(if $($(PKG)_PATCH),
		@$(call MESSAGE,"Download patch"))
	$(if $($(PKG)_PATCH),
		$(call DOWNLOAD,$($(PKG)_PATCH)))
endef

$(eval $(LINUX)_POST_DOWNLOAD_HOOKS += $(LINUX)_DOWNLOAD_PATCH)

define $(LINUX)_DOWNLOAD_PATCHES
	$(if $(LINUX_PATCHES),
		@$(call MESSAGE,"Download additional patches"))
	$(foreach patch,$(filter ftp://% http://%,$(LINUX_PATCHES)),\
		$(call DOWNLOAD,$(patch))$(sep))
endef

$(eval $(LINUX)_POST_DOWNLOAD_HOOKS += $(LINUX)_DOWNLOAD_PATCHES)

define $(LINUX)_APPLY_PATCHES
	for p in $(LINUX_PATCHES) ; do \
		if echo $$p | grep -q -E "^ftp://|^http://" ; then \
			$(APPLY_PATCHES) $(@D) $(DL_DIR) `basename $$p` ; \
		elif test -d $$p ; then \
			$(APPLY_PATCHES) $(@D) $$p linux-\*.patch ; \
		else \
			$(APPLY_PATCHES) $(@D) `dirname $$p` `basename $$p` ; \
		fi \
	done
endef

$(eval $(LINUX)_POST_PATCH_HOOKS += $(LINUX)_APPLY_PATCHES)


define $(LINUX)_CONFIGURE_CMDS
	$(INSTALL) -m 0644 $($(PKG)_PKGDIR)/../dot-config-$(KERNEL_ARCH) $($(PKG)_KERNEL_ARCH_PATH)/configs/buildroot_defconfig
	$(if $($(PKG)_FLAVOUR),patch $($(PKG)_KERNEL_ARCH_PATH)/configs/buildroot_defconfig $($(PKG)_PKGDIR)/dot-config-$(KERNEL_ARCH).diff)
	+$(TARGET_MAKE_ENV) $(MAKE1) $($(PKG)_MAKE_FLAGS) -C $($(PKG)_DIR) buildroot_defconfig
	rm $($(PKG)_KERNEL_ARCH_PATH)/configs/buildroot_defconfig
	$(if $(BR2_arm)$(BR2_armeb),
		$(call KCONFIG_ENABLE_OPT,CONFIG_AEABI,$($(PKG)_DIR)/.config))
	$(if $(BR2_TARGET_ROOTFS_CPIO),
		$(call KCONFIG_ENABLE_OPT,CONFIG_BLK_DEV_INITRD,$($(PKG)_DIR)/.config))
	# As the kernel gets compiled before root filesystems are
	# built, we create a fake cpio file. It'll be
	# replaced later by the real cpio archive, and the kernel will be
	# rebuilt using the linux-rebuild-with-initramfs target.
	$(if $(BR2_TARGET_ROOTFS_INITRAMFS),
		touch $(BINARIES_DIR)/rootfs.cpio
		$(call KCONFIG_SET_OPT,CONFIG_INITRAMFS_SOURCE,"$(BINARIES_DIR)/rootfs.cpio",$($(PKG)_DIR)/.config)
		$(call KCONFIG_SET_OPT,CONFIG_INITRAMFS_ROOT_UID,0,$($(PKG)_DIR)/.config)
		$(call KCONFIG_SET_OPT,CONFIG_INITRAMFS_ROOT_GID,0,$($(PKG)_DIR)/.config))
	$(if $(BR2_ROOTFS_DEVICE_CREATION_STATIC),,
		$(call KCONFIG_ENABLE_OPT,CONFIG_DEVTMPFS,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_DEVTMPFS_MOUNT,$($(PKG)_DIR)/.config))
	$(if $(BR2_ROOTFS_DEVICE_CREATION_DYNAMIC_EUDEV),
		$(call KCONFIG_ENABLE_OPT,CONFIG_INOTIFY_USER,$($(PKG)_DIR)/.config))
	$(if $(BR2_PACKAGE_KTAP),
		$(call KCONFIG_ENABLE_OPT,CONFIG_DEBUG_FS,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_EVENT_TRACING,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_PERF_EVENTS,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_FUNCTION_TRACER,$($(PKG)_DIR)/.config))
	$(if $(BR2_PACKAGE_SYSTEMD),
		$(call KCONFIG_ENABLE_OPT,CONFIG_CGROUPS,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_INOTIFY_USER,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_FHANDLE,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_AUTOFS4_FS,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_TMPFS_POSIX_ACL,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_TMPFS_POSIX_XATTR,$($(PKG)_DIR)/.config))
	$(if $(BR2_PACKAGE_SMACK),
		$(call KCONFIG_ENABLE_OPT,CONFIG_SECURITY,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_SECURITY_SMACK,$($(PKG)_DIR)/.config)
		$(call KCONFIG_ENABLE_OPT,CONFIG_SECURITY_NETWORK,$($(PKG)_DIR)/.config))
	$(if $(BR2_LINUX_KERNEL_APPENDED_DTB),
		$(call KCONFIG_ENABLE_OPT,CONFIG_ARM_APPENDED_DTB,$($(PKG)_DIR)/.config))
	+yes '' | $(TARGET_MAKE_ENV) $(MAKE1) $($(PKG)_MAKE_FLAGS) -C $($(PKG)_DIR) oldconfig
endef

ifeq ($(BR2_LINUX_KERNEL_DTS_SUPPORT),y)
ifeq ($(BR2_LINUX_KERNEL_DTB_IS_SELF_BUILT),)
define $(LINUX)_BUILD_DTB
	+$(TARGET_MAKE_ENV) $(MAKE) $($(PKG)_MAKE_FLAGS) -C $(@D) $(KERNEL_DTBS)
endef
define $(LINUX)_INSTALL_DTB
	# dtbs moved from arch/<ARCH>/boot to arch/<ARCH>/boot/dts since 3.8-rc1
	cp $(addprefix \
		$($(PKG)_KERNEL_ARCH_PATH)/boot/$(if $(wildcard \
		$(addprefix $($(PKG)_KERNEL_ARCH_PATH)/boot/dts/,$(KERNEL_DTBS))),dts/),$(KERNEL_DTBS)) \
		$(BINARIES_DIR)/
endef
define $(LINUX)_INSTALL_DTB_TARGET
	# dtbs moved from arch/<ARCH>/boot to arch/<ARCH>/boot/dts since 3.8-rc1
	cp $(addprefix \
		$($(PKG)_KERNEL_ARCH_PATH)/boot/$(if $(wildcard \
		$(addprefix $($(PKG)_KERNEL_ARCH_PATH)/boot/dts/,$(KERNEL_DTBS))),dts/),$(KERNEL_DTBS)) \
		$(TARGET_DIR)/boot/
endef
endif
endif

ifeq ($(BR2_LINUX_KERNEL_APPENDED_DTB),y)
# dtbs moved from arch/$ARCH/boot to arch/$ARCH/boot/dts since 3.8-rc1
define $(LINUX)_APPEND_DTB
	if [ -e $($(PKG)_KERNEL_ARCH_PATH)/boot/$(KERNEL_DTS_NAME).dtb ]; then \
		cat $($(PKG)_KERNEL_ARCH_PATH)/boot/$(KERNEL_DTS_NAME).dtb; \
	else \
		cat $($(PKG)_KERNEL_ARCH_PATH)/boot/dts/$(KERNEL_DTS_NAME).dtb; \
	fi >> $($(PKG)_KERNEL_ARCH_PATH)/boot/zImage
endef
ifeq ($(BR2_LINUX_KERNEL_APPENDED_UIMAGE),y)
# We need to generate a new u-boot image that takes into
# account the extra-size added by the device tree at the end
# of the image. To do so, we first need to retrieve both load
# address and entry point for the kernel from the already
# generate uboot image before using mkimage -l.
$(LINUX)_APPEND_DTB += $(sep) MKIMAGE_ARGS=`$(MKIMAGE) -l $($(PKG)_IMAGE_PATH) |\
	sed -n -e 's/Image Name:[ ]*\(.*\)/-n \1/p' -e 's/Load Address:/-a/p' -e 's/Entry Point:/-e/p'`; \
	$(MKIMAGE) -A $(MKIMAGE_ARCH) -O linux \
	-T kernel -C none $${MKIMAGE_ARGS} \
	-d $($(PKG)_KERNEL_ARCH_PATH)/boot/zImage $($(PKG)_IMAGE_PATH);
endif
endif

# Compilation. We make sure the kernel gets rebuilt when the
# configuration has changed.
define $(LINUX)_BUILD_CMDS
	$(if $(BR2_LINUX_KERNEL_USE_CUSTOM_DTS),
		cp $(call qstrip,$(BR2_LINUX_KERNEL_CUSTOM_DTS_PATH)) $($(PKG)_KERNEL_ARCH_PATH)/boot/dts/)
	+$(TARGET_MAKE_ENV) $(MAKE) $($(PKG)_MAKE_FLAGS) -C $(@D) $($(PKG)_TARGET_NAME)
	+@if grep -q "CONFIG_MODULES=y" $(@D)/.config; then 	\
		$(TARGET_MAKE_ENV) $(MAKE) $($(PKG)_MAKE_FLAGS) -C $(@D) modules ;	\
	fi
	$(LINUX_BUILD_DTB)
	$(LINUX_APPEND_DTB)
endef


ifeq ($(BR2_LINUX_KERNEL_INSTALL_TARGET),y)
define $(LINUX)_INSTALL_KERNEL_IMAGE_TO_TARGET
	install -m 0644 -D $($(PKG)_IMAGE_PATH) $(TARGET_DIR)/boot/kernel-$($(LINUX)_VERSION)
	install -m 0644 -D $($(PKG)_DIR)/System.map $(TARGET_DIR)/boot/System.map-$($(LINUX)_VERSION)
	$($(PKG)_INSTALL_DTB_TARGET)
endef
endif


define $(LINUX)_INSTALL_HOST_TOOLS
	# Installing dtc (device tree compiler) as host tool, if selected
	if grep -q "CONFIG_DTC=y" $(@D)/.config; then 	\
		$(INSTALL) -D -m 0755 $(@D)/scripts/dtc/dtc $(HOST_DIR)/usr/bin/dtc ;	\
	fi
endef


define $(LINUX)_INSTALL_IMAGES_CMDS
	cp $($(PKG)_IMAGE_PATH) $(BINARIES_DIR)
	$($(PKG)_INSTALL_DTB)
endef

define $(LINUX)_INSTALL_TARGET_CMDS
	$($(PKG)_INSTALL_KERNEL_IMAGE_TO_TARGET)
	# Install modules and remove symbolic links pointing to build
	# directories, not relevant on the target
	+@if grep -q "CONFIG_MODULES=y" $($(LINUX)_DIR)/.config; then 	\
		$(TARGET_MAKE_ENV) $(MAKE1) $($(LINUX)_MAKE_FLAGS) -C $($(LINUX)_DIR) modules_install; \
		rm -f $(TARGET_DIR)/lib/modules/$($(LINUX)_VERSION_PROBED)/build ;		\
		rm -f $(TARGET_DIR)/lib/modules/$($(LINUX)_VERSION_PROBED)/source ;	\
	fi
	$($(PKG)_INSTALL_HOST_TOOLS)
endef

$(eval $(generic-package))

$($(LINUX)_TARGET_CONFIGURE): LINUX := $(LINUX)
$($(LINUX)_TARGET_BUILD): LINUX := $(LINUX)
$($(LINUX)_TARGET_INSTALL_TARGET_SANDBOX): LINUX := $(LINUX)

ifeq ($(BR2_$(LINUX)),y)

$(eval LINUX_DEPENDENCIES += $(call pkgname))
$(eval LINUX_$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))_DEPENDENCIES += $(call pkgname))

$($(LINUX)_NAME)-modules: PKG := $(LINUX)
$($(LINUX)_NAME)-modules: LINUX := $(LINUX)
$($(LINUX)_NAME)-modules:
	if [ -f $($(LINUX)_DIR)/.config -a -f $(TARGET_DIR)/boot/System.map-$($(LINUX)_VERSION) ]; then \
		grep -q "^CONFIG_HAVE_UNDERSCORE_SYMBOL_PREFIX=y" $($(LINUX)_DIR)/.config 2>/dev/null && \
			prefix="-P _" || prefix=; \
		$(HOST_DIR)/sbin/depmod -b $(TARGET_DIR) -ae -F $(TARGET_DIR)/boot/System.map-$($(LINUX)_VERSION) $($(LINUX)_VERSION) $$prefix; \
		if [ -f "$($(LINUX)_ADD2MODULESDEP)" ]; then \
			depfile=$(TARGET_DIR)/lib/modules/$($(LINUX)_VERSION)/modules.dep; \
			while read module deps; \
			do \
				if grep -q "^$$module" $$depfile; then \
					sed -i "s#^$$module.*#& $$deps#" $$depfile; \
				else \
					echo "$$module $$deps" >> $$depfile; \
				fi \
			done < "$($(LINUX)_ADD2MODULESDEP)"; \
		fi; \
		if [ -f "$($(LINUX)_ADD2MODULESALIAS)" ]; then \
			cat "$($(LINUX)_ADD2MODULESALIAS)" >> $(TARGET_DIR)/lib/modules/$($(LINUX)_VERSION)/modules.alias; \
		fi \
	fi

linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-modules: $($(LINUX)_NAME)-modules

linux-kernel-menuconfig linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-menuconfig: $($(LINUX)_NAME)-menuconfig
linux-kernel-xconfig linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-xconfig: $($(LINUX)_NAME)-xconfig
linux-kernel-gconfig linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-gconfig: $($(LINUX)_NAME)-gconfig
linux-kernel-nconfig linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-nconfig: $($(LINUX)_NAME)-nconfig

$($(LINUX)_NAME)-menuconfig $($(LINUX)_NAME)-xconfig $($(LINUX)_NAME)-gconfig $($(LINUX)_NAME)-nconfig: PKG := $(LINUX)
$($(LINUX)_NAME)-menuconfig $($(LINUX)_NAME)-xconfig $($(LINUX)_NAME)-gconfig $($(LINUX)_NAME)-nconfig: LINUX := $(LINUX)
$($(LINUX)_NAME)-menuconfig $($(LINUX)_NAME)-xconfig $($(LINUX)_NAME)-gconfig $($(LINUX)_NAME)-nconfig: $($(LINUX)_TARGET_PATCH)
	$($(PKG)_CONFIGURE_CMDS)
	+$(MAKE) $($(PKG)_MAKE_FLAGS) -C $($(PKG)_DIR) $(subst $($(PKG)_NAME)-,,$@)
ifneq ($($(LINUX)_FLAVOUR),)
# canonicalize configuration file
	sed -i "/^# Linux\//s/$($(LINUX)_VERSION)/$($(LINUX)_EXT_VERSION)/" $($(PKG)_DIR)/.config
endif
	rm -f $($(PKG)_DIR)/.stamp_{built,target_installed,images_installed}

linux-kernel-update-config linux-kernel-$(call UPPERCASE,$($(LINUX)_MAJOR_VERSION))-update-config: $($(LINUX)_NAME)-update-config

$($(LINUX)_NAME)-update-config: PKG := $(LINUX)
$($(LINUX)_NAME)-update-config: LINUX := $(LINUX)
$($(LINUX)_NAME)-update-config:
ifeq ($($(LINUX)_FLAVOUR),)
	cp -f $($(PKG)_DIR)/.config $($(PKG)_PKGDIR)/../dot-config-$(KERNEL_ARCH)
ifneq ($(KCONFIG_FILE_DIR),)
	cp -f $($(PKG)_DIR)/.config $(KCONFIG_FILE_DIR)$($(PKG)_PKGDIR)/../dot-config-$(KERNEL_ARCH)
endif
else
	-diff -U 1 $($(PKG)_PKGDIR)/../dot-config-$(KERNEL_ARCH) $($(PKG)_DIR)/.config > $($(PKG)_PKGDIR)/dot-config-$(KERNEL_ARCH).diff
ifneq ($(KCONFIG_FILE_DIR),)
	cp -f $($(PKG)_PKGDIR)/dot-config-$(KERNEL_ARCH).diff $(KCONFIG_FILE_DIR)$($(PKG)_PKGDIR)/dot-config-$(KERNEL_ARCH).diff
endif
endif

# Support for rebuilding the kernel after the cpio archive has
# been generated in $(BINARIES_DIR)/rootfs.cpio.
$($(LINUX)_DIR)/.stamp_initramfs_rebuilt: PKG := $(LINUX)
$($(LINUX)_DIR)/.stamp_initramfs_rebuilt: LINUX := $(LINUX)
$($(LINUX)_DIR)/.stamp_initramfs_rebuilt: $($(LINUX)_DIR)/.stamp_target_installed $($(LINUX)_DIR)/.stamp_images_installed $(BINARIES_DIR)/rootfs.cpio
	@$(call MESSAGE,"Rebuilding kernel with initramfs")
	# Build the kernel.
	+$(TARGET_MAKE_ENV) $(MAKE) $($(PKG)_MAKE_FLAGS) -C $(@D) $($(PKG)_TARGET_NAME)
	$($(PKG)_APPEND_DTB)
	# Copy the kernel image to its final destination
	cp $($(PKG)_IMAGE_PATH) $(BINARIES_DIR)
	# If there is a .ub file copy it to the final destination
	test ! -f $($(PKG)_IMAGE_PATH).ub || cp $($(PKG)_IMAGE_PATH).ub $(BINARIES_DIR)
	$(Q)touch $@

# The initramfs building code must make sure this target gets called
# after it generated the initramfs list of files.
$($(LINUX)_NAME)-rebuild-with-initramfs: $($(LINUX)_DIR)/.stamp_initramfs_rebuilt

.PHONY: $($(LINUX)_NAME)-menuconfig $($(LINUX)_NAME)-xconfig $($(LINUX)_NAME)-gconfig $($(LINUX)_NAME)-nconfig

endif

LINUX:=
