/**
 * \file circd.c
 * \brief main program
 * \version $Revision:$
 * \date Last change: $Date:$
 * \author Oliver Dawid
 * \author Last change by: $Author:$
 */

/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2014 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */

#include "circd.h"

/// signal handler for children processes which have been done
static void sig_child(int sig);
/// signal handler for SIGTERM and SIGINT
static void sig_exit(int sig);
/// signal handler for closed pipes
static void sig_pipe(int sig);
/// signal handler for reread config/ SIGHUP
static void sig_reload(int sig);
/// exit function which cleans up persistence etc.
static void exit_circd(void);
/// init function which initializes all types of configured circuits
static void init_circuits(void);

/** main function
 * containing following steps:
 * 1. daemonize
 * 2. initialize signal handler
 * 3. read configuration from config directory
 * 4. initialize circuit types
 * 5. initialize listen port for clients
 * 6. check for lcr and non lcr circuits and switch them on
 * 7. main loop - do everything else in scheduler function poll_events()
 */
int main (int argc, char *argv[])
{
#if 0
#ifndef DEBUG
	if (daemon(0, 0) == -1) {
		perror("daemon() failed - cannot detach from tty");
	}
#endif
#endif
	signal(SIGCHLD, sig_child);
	signal(SIGTERM, sig_exit);
	signal(SIGINT, sig_exit);
	signal(SIGPIPE, sig_pipe);
	
	read_config();
	init_circuits();
	init_connection();
	
	current_lcr_mode = cm_automatic;

	check_lcr_circuits();
	check_nonlcr_circuits();

	for (;;)
		poll_events();

	return 0;
}

static void sig_child(int sig)
{
	pid_t pid;
	int status;
	
	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
		/* ... */
	}
}

static void sig_exit(int sig)
{
	exit_circd();
}

static void sig_pipe(int sig)
{
}

static void sig_reload(int sig)
{
}

static void exit_circd(void)
{
	exit(0);
}

static void init_circuits(void)
{
	list_head *le;
	circuit_t *cp;
	
	list_for_each(le, &circuits) {
		cp = list_entry(le, circuit_t, list);
		circuit_init(cp);
	}
}
