/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2014 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */

#define __USE_XOPEN_EXTENDED

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>
#include <glob.h>
#include <shadow.h>
#include <time.h>
#include <crypt.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <sys/poll.h>
#include <sys/utsname.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <linux/isdn.h>

#include "list.h"
#include "circd_defines.h"
#include "circd_types.h"
#include "md5.h"

/* circd.c */

/* data.c */
extern list_head circuits;
extern list_head connections;
extern char *telmond_log_dir;
extern char *mgetty_log_dir;
extern char *mgetty_log_file;
extern char *circd_log_dir;
extern char *circd_admin_pass;
extern char *circd_pass;
extern int password_mode;
extern int dialmode;
extern int may_enable;
extern int may_reboot;
extern int may_route;
extern int may_dial;
extern circuit_t *current_circuit;
extern circ_mode_t current_lcr_mode;
extern char *type_str[];
extern char *lcrflag_str[];
extern char *circmode_str[];
extern char *dialmode_str[];
extern char *weekday_str[];
extern struct _action_str_t action_str[];

extern struct timeval last_time_measured;
extern struct timeval current_time;

/* connection.c */
int init_connection(void);

/* dsl.c */
extern type_functable_t pppoe_tab;
extern type_functable_t pptp_tab;

/* isdn.c */
extern type_functable_t ippp_tab;
extern type_functable_t isdn_tab;

/* eth.c */
extern type_functable_t eth_tab;

/* ppp.c */
extern type_functable_t ppp_tab;

/* vpn.c */
extern type_functable_t vpn_tab;

/* events.c */
int register_event(int fd, int event, action_func af, void *context);
int unregister_event(int fd, int event);
int poll_events(void);
int set_timer(int usecs, timer_callback cb, void *context);
void clear_timer(timer_callback cb, void *context);

/* syslog.c */
void circd_syslog (int log_level, const char *fmt, ...);

/* config.c */
int read_config(void);

/* circuit.c */
extern type_functable_t unknown_tab;
circuit_t *find_circuit(char *name);
circuit_t *find_lcr_circuit(int day, int hour, int *tab_index);
void check_lcr_circuits(void);
void check_nonlcr_circuits(void);
void circuits_calc_rates(void);
void circuit_init(circuit_t *circ);
void circuit_start(request_t *rp);
void circuit_dial(request_t *rp);
void circuit_hangup(request_t *rp);
void circuit_stop(request_t *rp);
void circuit_pause(request_t *rp);
void circuit_cont(request_t *rp);
void circuit_dialmode(request_t *rp);
void circuit_hup_timeout(request_t *rp);
void circuit_status(request_t *rp);
void circuit_read_persistence(circuit_t *circ);
void circuit_write_persistence(circuit_t *circ);
void check_for_requests(void);
void remove_request(request_t *rp);

/* date.c */
int is_holiday(int day, int month, int year);

/* circ_common.c */
void circ_init(circuit_t *circ);
void circ_start(request_t *rp);
void circ_dial(request_t *rp);
void circ_hangup(request_t *rp);
void circ_stop(request_t *rp);
void circ_pause(request_t *rp);
void circ_cont(request_t *rp);
void circ_dialmode(request_t *rp);
void circ_hup_timeout(request_t *rp);
void circ_status(request_t *rp);
void circ_calc_troughput(circuit_t *circ);
void circ_read_persistence(circuit_t *circ);
void circ_write_persistence(circuit_t *circ);

/* iobytes.c */
device_t *find_device(char *name);
device_t *create_device(char *name);
void reset_device_accounting(device_t *dp);

/* usercmd.c */
char *parse_command(conn_t *client, char * buf);

