/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2014 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */

typedef enum {
	ct_pppoe,
	ct_ippp,
	ct_isdn,
	ct_pptp,
	ct_ppp,
	ct_eth,
	ct_vpn,
	ct_unknown
} circ_type_t;

typedef enum {
	cs_disabled,
	cs_starting,
	cs_offline,
	cs_dialing,
	cs_online,
	cs_pausing,
	cs_hangingup
} circ_status_t;

typedef enum {
	cm_disable,
	cm_manual,
	cm_automatic,
	cm_start,
	cm_dial,
	cm_pause,
	cm_cont
} circ_mode_t;

typedef enum {
	cf_disable,
	cf_enable,
	cf_active
} circ_lcrflag_t;

typedef enum {
	rs_queued,
	rs_pending
} request_state_t;

typedef union {
	circ_mode_t mode;
	time_t hup_timeout;
} request_data_t;

typedef struct _device_t {
	list_head list;
	struct _device_t *next;
	u_int32_t ibytes_low;
	u_int32_t obytes_low;
	u_int32_t ibytes_high;
	u_int32_t obytes_high;
	time_t last_read;
	float irate;
	float orate;
	char name[1];
} device_t;

typedef struct {
	char tab[21];    /* (HOURS_PER_DAY * DAYS_PER_WEEK + 7) / sizeof(char) */
	u_int32_t charge_int;
	double charge_p_int;
	union {
		circ_mode_t mode;    /* non-lcr mode */
		circ_lcrflag_t flag; /* lcr flag */
	} u;
	time_t sec;
	time_t lastaccess;
} time_table_t;

typedef struct {
	u_int32_t size;
	list_head list;
	list_head circ_list;
	list_head pending_requests;
	char *name;
	char *device;
	char *phone_in;
	char *phone_out;
	u_int32_t type;
	u_int32_t bandwidth_time;
	u_int32_t bandwidth_rate;
	u_int32_t hup_timeout;
	u_int32_t lcr;
	int32_t current_tab;
	u_int32_t start_timeout;
	u_int32_t dial_timeout;
	circ_status_t current_status;
	u_int32_t time_tab_count;
	device_t *dev;
	char type_str[16];
	time_table_t *time_tab;
} circuit_t;

struct _conn_t;

typedef struct _conn_t {
	list_head list;
	char inbuf[1024];
	char outbuf[1024];
	char inp;
	char outp;
	char outlen;

	int fd;
	
	char *ip;
	int password_state;
	char salt[2];
} conn_t;

typedef void (*action_func)(int fd, void *context);
typedef void (*cb_request)(int success, char *answer, void *context);

typedef struct {
	list_head list;
	int fd;
	int events;
	int tab_index;
	action_func read_func;
	action_func write_func;
	void *read_context;
	void *write_context;
} event_handler_t;

struct _request_t;
typedef void (*circuit_func)(struct _request_t *rp);

typedef struct _request_t {
	list_head list;
	int in_fd;
	int out_fd;
	circuit_func func;
	circuit_t *circ;
	request_data_t data;
	time_t timeout;
	struct _request_t *depend;
	cb_request cb;
	void *context;
	request_state_t state;
} request_t;

typedef struct {
	void (*init)(circuit_t *circ);
	void (*start)(request_t *rp);
	void (*dial)(request_t *rp);
	void (*hangup)(request_t *rp);
	void (*stop)(request_t *rp);
	void (*pause)(request_t *rp);
	void (*cont)(request_t *rp);
	void (*dialmode)(request_t *rp, circ_mode_t mode);
	void (*hup_timeout)(request_t *rp, time_t timeout);
	void (*status)(request_t *rp);
	void (*calc_rates)(circuit_t *circ);
	void (*calc_iobytes)(circuit_t *circ);
	void (*read_persistence)(circuit_t *circ);
	void (*write_persistence)(circuit_t *circ);
} type_functable_t;

typedef void (*timer_callback)(void *context);

struct _action_str_t {
	void *action;
	char *name;
};
