/**
 * \file connection.c
 * \brief telnet user interface to control the server
 * \version $Revision:$
 * \date Last change: $Date:$
 * \author Oliver Dawid
 * \author Last change by: $Author:$
 */

/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2014 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */

#include "circd.h"

static void accept_connection(int fd, void *context);
static void read_connection(int fd, void *context);
static void close_connection(int fd, conn_t *c);
static int sock_fd = -1;

int init_connection(void)
{
	struct sockaddr_in sa;
	socklen_t len = sizeof(sa);
	int fd;
	int i = 1;
	
	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		circd_syslog(LOG_ERR, "socket failed: %s", strerror(errno));
		return -1;
	}
	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *)&i, sizeof(i)) == -1) {
		circd_syslog(LOG_ERR, "setsockopt failed: %s", strerror(errno));
		return -1;
	}
	if (fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK | FD_CLOEXEC) == -1) {
		circd_syslog(LOG_ERR, "fcntl failed: %s", strerror(errno));
		close(fd);
		return -1;
	}
	memset(&sa, 0, sizeof(sa));
	inet_pton(AF_INET, "0.0.0.0", &sa.sin_addr.s_addr);
	sa.sin_port = htons(IMOND_PORT);
	if (bind(fd, (struct sockaddr *)&sa, len) == -1) {
		circd_syslog(LOG_ERR, "bind failed: %s", strerror(errno));
		return -1;
	}
	if (listen(fd, 5) == -1) {
		circd_syslog(LOG_ERR, "listen failed: %s", strerror(errno));
		return -1;
	}
	register_event(fd, POLLIN, accept_connection, NULL);
	sock_fd = fd;
	
	return 0;
}

static void accept_connection(int fd, void *context)
{
	int new_fd;
	struct sockaddr_in sa;
	socklen_t len = sizeof(sa);
	conn_t *c;
		
	if ((new_fd = accept(fd, (struct sockaddr *)&sa, &len)) == -1) {
		circd_syslog(LOG_ERR, "accept failed: %s", strerror(errno));
		return;
	}
	if (fcntl(new_fd, F_SETFL, fcntl(new_fd, F_GETFL, 0) | O_NONBLOCK | FD_CLOEXEC) == -1) {
		circd_syslog(LOG_ERR, "fcntl failed: %s", strerror(errno));
		close(new_fd);
		return;
	}
	if ((c = (conn_t *)calloc(1, sizeof(*c))) == NULL) {
		circd_syslog(LOG_ERR, "calloc failed: %s", strerror(errno));
		close(new_fd);
		return;
	}
	c->password_state = PASSWORD_STATE_ADMIN | PASSWORD_STATE_NORMAL;
	register_event(new_fd, POLLIN, read_connection, c);
}

static void read_connection(int fd, void *context)
{
	size_t len;
	conn_t *c = (conn_t *)context;
	char *outmsg;
	
	if ((len = read(fd, c->inbuf, sizeof(c->inbuf))) < 1) {
		if (errno != EWOULDBLOCK) {
			circd_syslog(LOG_ERR, "read failed: %s", strerror(errno));
			close_connection(fd, c);
		}
	}
	outmsg = parse_command(c, c->inbuf);
	if (outmsg)
		write(fd, outmsg, strlen(outmsg));
}

static void close_connection(int fd, conn_t *c)
{
	unregister_event(fd, POLLIN | POLLOUT);
	close(fd);
}
