/*****************************************************************************
 *  @file grammar/varass_parser.y
 *  Parser for files containing variable assignments.
 *
 *  Copyright (c) 2012 The fli4l team
 *
 *  This file is part of fli4l.
 *
 *  fli4l is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  fli4l is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with fli4l.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Last Update: $Id: varass_parser.y 23640 2012-09-11 08:21:24Z kristov $
 *****************************************************************************
 */

%pure-parser
%lex-param   {void *scanner}
%parse-param {void *scanner}
%parse-param {struct package_file_t *file}
%parse-param {struct varass_parse_tree_t *tree}
%parse-param {int priority}
%locations
%error-verbose

%{
#include "grammar/varass.h"
%}

%union {
    char *str;
    struct qstr_t *qstr;
    struct identifier_t *id;
    struct varass_t *assignment;
}

%destructor { free($$); } <str>
%destructor { qstr_destroy($$); } <qstr>
%destructor { identifier_destroy($$); } <id>
%destructor { varass_destroy($$); } <assignment>

%{
#include <string.h>
#include <stdlib.h>
#include "grammar/varass_scanner.h"
#include "grammar/parser.h"

PARSER_DEFINE_ERROR_PROLOGUE(struct varass_parse_tree_t *)
int priority,
PARSER_DEFINE_ERROR_EPILOGUE("variable assignments")
%}

%token <id> ID
%token EQ
%token <qstr> QSTRING
%token <str> COMMENT
%token <str> UNKNOWN

%type <str> comment
%type <assignment> assignment

%%

/* file which contains a list of statements */
file:
  statements
;

/* statement list which may be empty */
statements:
  /* empty */
| statements statement
;

/* statement which is either an assignment or an error */
statement:
  assignment  { varass_parse_tree_add_assignment(tree, $1); }
| error       { tree->ok = FALSE; }
;

/* variable assignment */
assignment:
  ID EQ QSTRING comment
{
    struct location_t *location
        = location_create(file, @1.first_line, @1.first_column);
    $$ = varass_create($1, $3, priority, $4, location);
}
;

/* comment that does not begin a line */
comment:
  /* empty */ { $$ = strdup(""); }
| COMMENT     { $$ = $1; }
;
