/*!
 * @file    parser-demo-adsl.c
 * @brief   implementation file for demo parser
 * @details data generation for demo modem
 *
 * @author  Carsten Spie&szlig; fli4l@carsten-spiess.de
 * @date    20.02.2013
 *
 * @note
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * @n
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * @n
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "parser.h"
#include "log.h"

/*!
 * @def         RAND_INT(x)
 * @brief       integer random value
 * @def         RAND_DOUBLE(x)
 * @brief       double random value
 */
#if defined (CFG_TEST)
#define RAND_INT(x) 0
#define RAND_DOUBLE(x) 0
#else
#define RAND_INT(x) (x * random() / RAND_MAX)
#define RAND_DOUBLE(x) (x * (double) random() / (double) RAND_MAX)
#endif
#if defined (CFG_COMMAND)
/*!
 * @var         static int demo_bitalloc_up[]
 * @brief       bit allocation demo data
 */
static int demo_bitalloc_up[] =
{
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    11, 11, 12, 12, 13, 13, 13, 14, 14, 14, 14, 14, 13, 8, 0, 8, 13, 13, 13, 13, 13, 13, 13, 12, 12, 12, 12, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

/*!
 * @var         static int demo_bitalloc_down[]
 * @brief       bit allocation demo data
 */
static int demo_bitalloc_down[] =
{
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 14, 14, 15, 14, 15, 14, 14, 14, 14, 2, 15, 14, 14, 14, 15, 14, 14, 15, 14, 14, 15, 14, 14, 14, 14, 14, 15, 14, 14, 14, 14,
    13, 15, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 13, 13, 13, 13, 13, 13, 13, 13, 13, 14, 13, 13, 13, 14, 13, 13, 13,
    14, 13, 14, 13, 13, 14, 13, 13, 14, 14, 13, 13, 13, 13, 13, 13, 14, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 12, 13, 13,
    13, 12, 13, 13, 12, 7, 0, 7, 13, 12, 13, 12, 12, 13, 12, 12, 13, 12, 12, 12, 13, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 11, 12, 12, 12, 11, 11, 12, 12, 11, 11, 11, 12, 12, 12, 11, 11, 11, 11, 12, 11, 12, 11, 11, 12, 12, 11,
    12, 11, 12, 11, 12, 11, 12, 11, 12, 11, 12, 11, 12, 11, 11, 12, 12, 12, 11, 12, 11, 11, 12, 11, 11, 12, 12, 11, 12, 11, 11, 12,
    12, 12, 12, 11, 8, 0, 0, 8, 11, 12, 11, 12, 11, 12, 11, 12, 11, 12, 11, 12, 11, 11, 12, 12, 12, 11, 11, 12, 12, 12, 11, 12,
    12, 12, 11, 12, 12, 11, 12, 12, 12, 12, 12, 12, 12, 11, 11, 11, 11, 11, 11, 12, 11, 12, 11, 11, 11, 12, 11, 11, 11, 11, 11, 11,
    11, 11, 11, 11, 10, 11, 10, 11, 11, 10, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 10, 11, 11, 11, 11, 11, 11, 11, 11, 10,
    11, 11, 11, 10, 10, 11, 11, 10, 11, 9, 10, 11, 10, 11, 11, 11, 11, 11, 11, 10, 11, 11, 11, 10, 10, 11, 10, 11, 11, 10, 11, 11,
    10, 11, 11, 10, 10, 11, 10, 10, 10, 11, 10, 11, 10, 11, 10, 11, 11, 11, 11, 10, 11, 10, 11, 10, 11, 11, 11, 11, 11, 10, 11, 10,
    11, 11, 10, 11, 10, 11, 10, 11, 11, 11, 10, 10, 11, 11, 11, 10, 10, 11, 10, 11, 11, 10, 10, 11, 10, 10, 10, 11, 10, 10, 10, 11,
    10, 10, 10, 10, 10, 10, 9, 9, 9, 9, 10, 10, 10, 9, 10, 10, 9, 10, 10, 10, 9, 9, 10, 9, 9, 9, 9, 9, 9, 9, 9, 9,
    9, 8, 9, 9, 8, 8, 8, 9, 8, 8, 8, 7, 8, 7, 8, 7, 7, 7, 7, 7, 6, 7, 6, 6, 6, 5, 6, 5, 5, 0, 0, 0
};

/*!
 * @var         static int demo_snr[]
 * @brief       SNR demo data
 */
static int demo_snr[] =
{
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 39, 41,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 48, 48, 48, 48, 48, 48, 48, 47, 47, 48, 47, 47, 48, 48, 48, 47, 48, 48, 48, 48, 48, 47, 48, 47, 47, 47, 47, 48, 48, 48, 49,
    47, 48, 48, 48, 48, 48, 47, 48, 48, 48, 47, 47, 47, 47, 47, 47, 47, 46, 47, 46, 47, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46,
    46, 45, 46, 45, 45, 45, 45, 45, 45, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 44, 43, 43, 43, 43, 43, 43, 43,
    43, 43, 43, 43, 42, 42, 42, 42, 42, 42, 42, 42, 42, 42, 42, 42, 42, 42, 41, 41, 42, 41, 41, 41, 41, 41, 40, 42, 41, 41, 41, 41,
    41, 41, 41, 40, 41, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 39,
    39, 39, 40, 39, 39, 40, 39, 40, 40, 39, 39, 39, 39, 39, 39, 39, 38, 39, 39, 39, 39, 39, 39, 39, 39, 39, 38, 39, 39, 39, 39, 41,
    41, 41, 39, 39, 39, 39, 39, 39, 39, 39, 40, 39, 39, 39, 40, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39,
    39, 39, 39, 39, 39, 38, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 38, 38, 38, 38, 39, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38,
    38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 37, 38, 38, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 38, 37, 37, 37,
    37, 37, 37, 37, 37, 37, 37, 37, 38, 37, 37, 37, 36, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 36, 36, 36, 36, 36,
    36, 37, 36, 36, 36, 36, 36, 36, 37, 36, 36, 36, 36, 37, 36, 36, 36, 36, 36, 36, 37, 36, 36, 37, 36, 36, 36, 36, 36, 36, 36, 36,
    36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 35, 36, 35, 35,
    35, 35, 35, 35, 35, 35, 35, 35, 34, 34, 34, 34, 34, 34, 34, 33, 34, 33, 33, 33, 33, 33, 33, 32, 32, 32, 32, 32, 32, 32, 31, 31,
    31, 31, 31, 30, 30, 30, 30, 29, 29, 29, 28, 28, 28, 27, 27, 27, 26, 26, 25, 25, 24, 24, 23, 23, 22, 21, 21, 20, 20, 0, 0, 0
};

/*!
 * @var         static int demo_gainQ2[]
 * @brief       gain Q2 demo data
 */
static int demo_gainQ2[] =
{
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    109, 88, 99, 87, 105, 96, 90, 118, 112, 109, 110, 110, 83, 86, 91, 96, 106, 108, 109, 108, 108, 109, 112, 87, 95, 105, 111, 0, 0, 0, 0, 0,
    0, 84, 84, 115, 86, 122, 86, 86, 91, 94, 100, 129, 94, 89, 86, 122, 91, 89, 122, 84, 81, 118, 91, 89, 94, 97, 94, 129, 89, 89, 115, 112,
    100, 91, 86, 86, 125, 91, 94, 89, 89, 125, 91, 94, 91, 94, 94, 100, 100, 102, 102, 105, 97, 105, 102, 105, 108, 105, 75, 105, 102, 105, 102, 77,
    115, 84, 112, 86, 86, 122, 86, 86, 125, 94, 91, 94, 91, 94, 94, 97, 133, 94, 97, 100, 100, 100, 97, 100, 97, 102, 105, 105, 102, 75, 105, 105,
    108, 77, 108, 112, 81, 115, 84, 89, 118, 81, 118, 84, 84, 118, 84, 86, 122, 89, 91, 91, 122, 91, 100, 91, 94, 97, 105, 122, 94, 97, 100, 94,
    100, 102, 102, 102, 102, 102, 75, 108, 102, 108, 77, 105, 105, 105, 79, 112, 79, 112, 105, 105, 79, 112, 79, 115, 112, 77, 112, 105, 79, 108, 108, 81,
    115, 81, 112, 84, 115, 79, 115, 79, 112, 86, 115, 84, 115, 86, 84, 122, 94, 91, 86, 125, 89, 84, 118, 81, 86, 118, 94, 89, 122, 86, 84, 97,
    94, 133, 86, 84, 115, 81, 122, 89, 81, 118, 79, 115, 84, 115, 79, 115, 86, 118, 84, 115, 84, 81, 118, 86, 118, 84, 86, 118, 86, 115, 84, 86,
    122, 89, 89, 122, 86, 91, 86, 122, 84, 89, 118, 89, 122, 89, 84, 89, 129, 91, 97, 94, 91, 91, 97, 100, 94, 133, 94, 97, 94, 97, 97, 94,
    97, 97, 97, 97, 97, 100, 97, 97, 100, 97, 102, 102, 100, 100, 102, 102, 105, 105, 105, 105, 108, 102, 75, 105, 102, 105, 79, 105, 100, 105, 105, 75,
    105, 105, 108, 108, 72, 105, 108, 79, 102, 115, 77, 108, 115, 77, 105, 108, 75, 108, 112, 77, 108, 108, 102, 75, 108, 112, 79, 84, 122, 84, 118, 86,
    86, 112, 115, 84, 84, 115, 81, 115, 79, 118, 81, 118, 81, 115, 86, 86, 122, 84, 118, 81, 112, 84, 115, 79, 118, 86, 84, 122, 86, 86, 122, 81,
    86, 118, 84, 115, 81, 115, 81, 118, 81, 118, 84, 81, 118, 86, 115, 84, 86, 122, 86, 89, 122, 86, 86, 125, 89, 86, 89, 129, 91, 89, 97, 94,
    97, 94, 97, 97, 97, 97, 100, 100, 102, 102, 105, 105, 108, 108, 108, 84, 112, 84, 84, 122, 84, 86, 125, 91, 91, 91, 97, 97, 100, 100, 105, 105,
    108, 79, 112, 115, 84, 86, 89, 129, 97, 100, 102, 75, 112, 81, 122, 89, 94, 97, 105, 108, 81, 122, 91, 97, 105, 81, 118, 91, 91, 0, 0, 0
};

/*!
 * @var         static int demo_noisemargin[]
 * @brief       noisemargin demo data
 */
static int demo_noisemargin[] =
{
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 32, 2, 2, 2, 3, 1, 2, 2, 2, 2, 1, 1, 3, 2, 2, 1, 1, 1, 2, 2, 2, 2,
    3, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 2, 2, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 1, 2, 1, 1, 3,
    2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 2, 1, 1, 1, 2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 1, 1, 2,
    2, 2, 1, 2, 2, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 1, 2, 2, 1,
    2, 2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 3, 3, 2, 2, 2, 3, 3, 2, 3, 2, 2, 2,
    3, 2, 3, 2, 2, 3, 2, 2, 2, 2, 3, 2, 3, 3, 2, 2, 3, 2, 2, 2, 2, 3, 2, 2, 2, 2, 3, 2, 3, 3, 2, 2,
    2, 3, 3, 3, 3, 3, 2, 2, 3, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 3, 2, 3, 2, 3, 3, 3,
    2, 3, 3, 2, 3, 3, 3, 2, 3, 3, 2, 2, 2, 2, 3, 3, 2, 2, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 2,
    2, 2, 3, 3, 3, 2, 3, 2, 3, 3, 3, 2, 3, 3, 3, 2, 3, 2, 3, 3, 2, 3, 2, 2, 2, 3, 2, 2, 2, 2, 3, 3,
    2, 2, 3, 3, 3, 3, 3, 2, 2, 3, 3, 3, 2, 2, 2, 3, 3, 2, 3, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 3, 3, 2, 3, 3, 3, 3, 3, 3, 2, 3, 3, 3, 3, 3,
    3, 3, 2, 3, 2, 3, 3, 2, 3, 3, 3, 2, 3, 3, 3, 3, 3, 3, 3, 3, 2, 3, 3, 3, 2, 2, 3, 2, 3, 0, 0, 0
};

/*!
 * @var         static int demo_chancharlog[]
 * @brief       channel characteristics demo data
 */
static int demo_chancharlog[] =
{
    100, 45, 48, 48, 51, 57, 51, 51, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 100, 57, 57, 57, 57, 57, 57, 57, 57, 57,
    57, 58, 100, 100, 100, 62, 63, 100, 66, 67, 68, 69, 66, 71, 66, 63, 62, 57, 54, 52, 49, 47, 45, 43, 42, 40, 38, 36, 35, 34, 32, 31,
    30, 29, 27, 26, 25, 24, 23, 22, 21, 21, 20, 19, 19, 19, 19, 19, 19, 20, 20, 20, 21, 21, 21, 22, 22, 22, 22, 23, 23, 23, 23, 23,
    24, 24, 24, 24, 24, 24, 24, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26,
    26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
    27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28,
    28, 28, 28, 28, 28, 28, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 30, 30, 30, 30, 30, 30, 30,
    30, 30, 30, 30, 30, 30, 30, 30, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 35,
    35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 35, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 37, 37,
    37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38,
    38, 38, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 40, 40, 40, 40, 40, 40, 40, 40,
    40, 40, 40, 40, 40, 40, 40, 40, 40, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 41, 42, 42, 42, 42, 42, 42, 42,
    42, 42, 42, 42, 42, 42, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 44, 44, 44, 44, 44, 44, 44, 44, 44, 45, 45, 45, 45, 45,
    45, 45, 45, 46, 46, 46, 46, 46, 46, 47, 47, 47, 47, 47, 47, 47, 48, 48, 48, 48, 48, 49, 49, 49, 49, 49, 50, 50, 50, 50, 50, 51,
    51, 51, 52, 52, 52, 52, 53, 53, 53, 53, 54, 54, 54, 55, 55, 55, 56, 56, 57, 57, 57, 57, 58, 58, 59, 59, 59, 60, 60, 60, 61, 947
};
#endif /* defined (CFG_COMMAND) */

void parser_init(int command, const char* username, const char* password)
{
#if !defined (CFG_COMMAND)
    (void) command;
#endif /* !defined (CFG_COMMAND) */
    (void) username;
    (void) password;

    _parser_init(command);

    strcpy(parser_data.modemstate.str, "up");
    parser_data.modemstate.state = 1;
    parser_data.modemstate.valid = 2;

    strcpy(parser_data.operationmode.str, "G.992.5 Annex B");
    parser_data.operationmode.valid = 1;
    parser_data.bandplan = &bandplan_ADSL2plus_AnnexB;

    strcpy(parser_data.channelmode.str, "Interleave Path");
    parser_data.channelmode.valid = 1;

    parser_data.atm.pvc.vpi = 1;
    parser_data.atm.pvc.vci = 32;
    parser_data.atm.pvc.valid = 2;

    memcpy(parser_data.atm.atu_c.vendor, "CENT", sizeof(parser_data.atm.atu_c.vendor));
    parser_data.atm.atu_c.vendspec[0] = 255;
    parser_data.atm.atu_c.vendspec[1] = 123;
    parser_data.atm.atu_c.revision = 2;
    parser_data.atm.atu_c.valid = 3;

    memcpy(parser_data.atm.atu_r.vendor, "TEST", sizeof(parser_data.atm.atu_r.vendor));
    parser_data.atm.atu_r.vendspec[0] = 0;
    parser_data.atm.atu_r.vendspec[1] = 0;
    parser_data.atm.atu_r.revision = 0;
    parser_data.atm.atu_r.valid = 3;


    parser_data.noisemargin[0].down = 3.0 + RAND_DOUBLE(1.0);
    parser_data.noisemargin[0].up = 4.0 + RAND_DOUBLE(1.0);;
    parser_data.noisemargin[0].valid = 2;
    parser_data.attenuation[0].down = 20.0 + RAND_DOUBLE(2.0);
    parser_data.attenuation[0].up = 15.0 + RAND_DOUBLE(2.0);
    parser_data.attenuation[0].valid = 2;
    parser_data.txpower.down = 21.0 + RAND_DOUBLE(1.0);
    parser_data.txpower.up = 14.0 + RAND_DOUBLE(1.0);
    parser_data.txpower.valid = 2;
    parser_data.bandwidth.cells.down = 23584;
    parser_data.bandwidth.cells.up = 2358;
    parser_data.bandwidth.cells.valid = 2;
    parser_data.bandwidth.kbit.down = 10000 + RAND_INT(100);
    parser_data.bandwidth.kbit.up = 1000 + RAND_INT(20);
    parser_data.bandwidth.kbit.valid = 2;

    parser_data.statistics.error.FEC.Rx = 123;
    parser_data.statistics.error.FEC.Tx = 987;
    parser_data.statistics.error.FEC.valid = 2;
    parser_data.statistics.error.CRC.Rx = 456;
    parser_data.statistics.error.CRC.Tx = 654;
    parser_data.statistics.error.CRC.valid = 2;
    parser_data.statistics.error.HEC.Rx = 789;
    parser_data.statistics.error.HEC.Tx = 321;
    parser_data.statistics.error.HEC.valid = 2;
    parser_data.statistics.failure.error_secs_15min = 15 + RAND_INT(10);
    parser_data.statistics.failure.error_secs_day = 1440 + RAND_INT(100);
    parser_data.statistics.failure.valid = 2;

#if defined (CFG_COMMAND)
    if (PARSER_INFO == command)
    {
        memcpy (parser_data.tone.bitalloc_up.data, demo_bitalloc_up, sizeof (demo_bitalloc_up));
        parser_data.tone.bitalloc_up.valid = 1;
        memcpy (parser_data.tone.bitalloc_down.data, demo_bitalloc_down, sizeof (demo_bitalloc_down));
        parser_data.tone.bitalloc_down.valid = 1;
        memcpy (parser_data.tone.snr.data, demo_snr, sizeof (demo_snr));
        parser_data.tone.snr.valid = 1;
        memcpy (parser_data.tone.gainQ2.data, demo_gainQ2, sizeof (demo_gainQ2));
        parser_data.tone.gainQ2.valid = 1;
        memcpy (parser_data.tone.noisemargin.data, demo_noisemargin, sizeof (demo_noisemargin));
        parser_data.tone.noisemargin.valid = 1;
        memcpy (parser_data.tone.chancharlog.data, demo_chancharlog, sizeof (demo_chancharlog));
        parser_data.tone.chancharlog.valid = 1;
    }
#endif /* defined (CFG_COMMAND) */
}

// _oOo_
