/*
 *  leds-wrap.c - LED support for PC-Engines WRAP board
 *
 *  Copyright (c) 2014  Carsten Spiess <fli4l at carsten-spiess.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/leds.h>
#include <linux/platform_device.h>

static struct gpio_led wrap_leds[] = {
	{
		.name = "wrap::1",
		.gpio = 2,
		.default_trigger = "none",
		.active_low = 1,
	},
	{
		.name = "wrap::2",
		.gpio = 3,
		.default_trigger = "none",
		.active_low = 1,
	},
	{
		.name = "wrap::3",
		.gpio = 18,
		.default_trigger = "none",
		.active_low = 1,
	},
};

static struct gpio_led_platform_data wrap_leds_data = {
	.num_leds = ARRAY_SIZE (wrap_leds),
	.leds = wrap_leds,
};

static void leds_wrap_release (struct device *dev)
{
}

static struct platform_device wrap_leds_dev = {
	.name = "leds-gpio",
	.id = -1,
	.dev = {
		.release = leds_wrap_release,
		.platform_data = &wrap_leds_data,
	}
};

static int __init leds_wrap_init (void)
{
	int ret;
	ret = platform_device_register (&wrap_leds_dev);
	if (ret) {
		pr_err ("adding leds device failed\n");
	}
	return ret;
}

static void __exit leds_wrap_exit (void)
{
	platform_device_unregister (&wrap_leds_dev);
}

MODULE_AUTHOR ("Carsten Spiess <fli4l at carsten-spiess.de>");
MODULE_DESCRIPTION ("GPIO-LED driver for PC-Engines WRAP");
MODULE_LICENSE ("GPL");


module_init (leds_wrap_init);
module_exit (leds_wrap_exit);


